@@ -10,6 +10,7 @@ use crate::chunked_array::cast::CastOptions;
 #[cfg(feature = "object")]
 use crate::chunked_array::object::PolarsObjectSafe;
 use crate::prelude::*;
+use crate::utils::{first_non_null, last_non_null};
 
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
@@ -453,6 +454,11 @@ pub trait SeriesTrait:
         polars_bail!(opq = arg_unique, self._dtype());
     }
 
+    /// Get dense ids for each unique value.
+    ///
+    /// Returns: (n_unique, unique_ids)
+    fn unique_id(&self) -> PolarsResult<(IdxSize, Vec<IdxSize>)>;
+
     /// Get a mask of the null values.
     fn is_null(&self) -> BooleanChunked;
 
@@ -554,6 +560,23 @@ pub trait SeriesTrait:
         Scalar::new(dt.clone(), av)
     }
 
+    /// Get the first non-null element of the [`Series`] as a [`Scalar`]
+    ///
+    /// If the [`Series`] is empty, a [`Scalar`] with a [`AnyValue::Null`] is returned.
+    fn first_non_null(&self) -> Scalar {
+        let av = if self.len() == 0 {
+            AnyValue::Null
+        } else {
+            let idx = if self.has_nulls() {
+                first_non_null(self.chunks().iter().map(|c| c.as_ref())).unwrap_or(0)
+            } else {
+                0
+            };
+            self.get(idx).map_or(AnyValue::Null, AnyValue::into_static)
+        };
+        Scalar::new(self.dtype().clone(), av)
+    }
+
     /// Get the last element of the [`Series`] as a [`Scalar`]
     ///
     /// If the [`Series`] is empty, a [`Scalar`] with a [`AnyValue::Null`] is returned.
@@ -569,6 +592,25 @@ pub trait SeriesTrait:
         Scalar::new(dt.clone(), av)
     }
 
+    /// Get the last non-null element of the [`Series`] as a [`Scalar`]
+    ///
+    /// If the [`Series`] is empty, a [`Scalar`] with a [`AnyValue::Null`] is returned.
+    fn last_non_null(&self) -> Scalar {
+        let n = self.len();
+        let av = if n == 0 {
+            AnyValue::Null
+        } else {
+            let idx = if self.has_nulls() {
+                last_non_null(self.chunks().iter().map(|c| c.as_ref()), n).unwrap_or(n - 1)
+            } else {
+                n - 1
+            };
+            // SAFETY: len-1 < len if len != 0
+            unsafe { self.get_unchecked(idx) }.into_static()
+        };
+        Scalar::new(self.dtype().clone(), av)
+    }
+
     #[cfg(feature = "approx_unique")]
     fn approx_n_unique(&self) -> PolarsResult<IdxSize> {
         polars_bail!(opq = approx_n_unique, self._dtype());
