@@ -2,12 +2,9 @@ use std::hash::BuildHasher;
 
 use arrow::bitmap::MutableBitmap;
 use either::Either;
-use parking_lot::{RwLock, RwLockWriteGuard};
+use parking_lot::RwLock;
 use polars::prelude::*;
 use polars_ffi::version_0::SeriesExport;
-#[cfg(feature = "pivot")]
-use polars_lazy::frame::pivot::{pivot, pivot_stable};
-use pyo3::IntoPyObjectExt;
 use pyo3::exceptions::PyIndexError;
 use pyo3::prelude::*;
 use pyo3::pybacked::PyBackedStr;
@@ -15,17 +12,13 @@ use pyo3::types::{PyList, PyType};
 
 use self::row_encode::{_get_rows_encoded_ca, _get_rows_encoded_ca_unordered};
 use super::PyDataFrame;
+use crate::PyLazyFrame;
 use crate::conversion::Wrap;
 use crate::error::PyPolarsErr;
-use crate::map::dataframe::{
-    apply_lambda_unknown, apply_lambda_with_bool_out_type, apply_lambda_with_primitive_out_type,
-    apply_lambda_with_string_out_type,
-};
 use crate::prelude::strings_to_pl_smallstr;
 use crate::py_modules::polars;
 use crate::series::{PySeries, ToPySeries, ToSeries};
 use crate::utils::EnterPolarsExt;
-use crate::{PyExpr, PyLazyFrame};
 
 #[pymethods]
 impl PyDataFrame {
@@ -411,41 +404,23 @@ impl PyDataFrame {
     pub fn unpivot(
         &self,
         py: Python<'_>,
-        on: Vec<PyBackedStr>,
+        on: Option<Vec<PyBackedStr>>,
         index: Vec<PyBackedStr>,
         value_name: Option<&str>,
         variable_name: Option<&str>,
     ) -> PyResult<Self> {
-        use polars_ops::pivot::UnpivotDF;
-        let args = UnpivotArgsIR {
-            on: strings_to_pl_smallstr(on),
-            index: strings_to_pl_smallstr(index),
-            value_name: value_name.map(|s| s.into()),
-            variable_name: variable_name.map(|s| s.into()),
-        };
+        use polars_ops::unpivot::UnpivotDF;
+        let args = UnpivotArgsIR::new(
+            self.df.read().get_column_names_owned(),
+            on.map(strings_to_pl_smallstr),
+            strings_to_pl_smallstr(index),
+            value_name.map(|s| s.into()),
+            variable_name.map(|s| s.into()),
+        );
 
         py.enter_polars_df(|| self.df.read().unpivot2(args))
     }
 
-    #[cfg(feature = "pivot")]
-    #[pyo3(signature = (on, index, values, maintain_order, sort_columns, aggregate_expr, separator))]
-    pub fn pivot_expr(
-        &self,
-        py: Python<'_>,
-        on: Vec<String>,
-        index: Option<Vec<String>>,
-        values: Option<Vec<String>>,
-        maintain_order: bool,
-        sort_columns: bool,
-        aggregate_expr: Option<PyExpr>,
-        separator: Option<&str>,
-    ) -> PyResult<Self> {
-        let df = self.df.read().clone(); // Clone to avoid dead lock on re-entrance in aggregate_expr.
-        let fun = if maintain_order { pivot_stable } else { pivot };
-        let agg_expr = aggregate_expr.map(|expr| expr.inner);
-        py.enter_polars_df(|| fun(&df, on, index, values, sort_columns, agg_expr, separator))
-    }
-
     pub fn partition_by(
         &self,
         py: Python<'_>,
@@ -492,38 +467,6 @@ impl PyDataFrame {
         py.enter_polars_df(|| Ok(self.df.read().null_count()))
     }
 
-    #[pyo3(signature = (lambda, output_type, inference_size))]
-    pub fn map_rows(
-        &self,
-        lambda: Bound<PyAny>,
-        output_type: Option<Wrap<DataType>>,
-        inference_size: usize,
-    ) -> PyResult<(Py<PyAny>, bool)> {
-        Python::attach(|py| {
-            let mut df = self.df.write();
-            df.as_single_chunk_par(); // needed for series iter
-            let df = &*RwLockWriteGuard::downgrade(df);
-
-            use apply_lambda_with_primitive_out_type as apply;
-            #[rustfmt::skip]
-            let out = match output_type.map(|dt| dt.0) {
-                Some(DataType::Int32) => apply::<Int32Type>(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::Int64) => apply::<Int64Type>(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::UInt32) => apply::<UInt32Type>(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::UInt64) => apply::<UInt64Type>(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::Float32) => apply::<Float32Type>(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::Float64) => apply::<Float64Type>(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::Date) => apply::<Int32Type>(df, py, lambda, 0, None)?.into_date().into_series(),
-                Some(DataType::Datetime(tu, tz)) => apply::<Int64Type>(df, py, lambda, 0, None)?.into_datetime(tu, tz).into_series(),
-                Some(DataType::Boolean) => apply_lambda_with_bool_out_type(df, py, lambda, 0, None)?.into_series(),
-                Some(DataType::String) => apply_lambda_with_string_out_type(df, py, lambda, 0, None)?.into_series(),
-                _ => return apply_lambda_unknown(df, py, lambda, inference_size),
-            };
-
-            Ok((PySeries::from(out).into_py_any(py)?, false))
-        })
-    }
-
     pub fn shrink_to_fit(&self, py: Python) -> PyResult<()> {
         py.enter_polars_ok(|| self.df.write().shrink_to_fit())
     }
