@@ -11,11 +11,14 @@ mod execute;
 pub use dispatch::build_streaming_query_executor;
 pub(crate) mod expression;
 mod graph;
+pub use graph::{GraphNodeKey, LogicalPipe, LogicalPipeKey};
 pub use skeleton::{QueryResult, StreamingQuery};
 mod metrics;
+pub use metrics::{GraphMetrics, NodeMetrics};
 mod morsel;
 mod nodes;
 mod physical_plan;
+pub use physical_plan::PhysNodeKey;
 #[cfg(feature = "physical_plan_visualization")]
 pub use physical_plan::visualization as physical_plan_visualization;
 mod pipe;
