@@ -183,8 +183,8 @@ def test_categorical_global_ordering(
     s2 = pl.Series("b_cat", ["a", "b", "c", "a", "c"], dtype=pl.Categorical)
     assert_series_equal(op(s, s2), expected_lexical)
 
-    s = s.cast(pl.Categorical("lexical"))
-    s2 = s2.cast(pl.Categorical("lexical"))
+    s = s.cast(pl.Categorical())
+    s2 = s2.cast(pl.Categorical())
     assert_series_equal(op(s, s2), expected_lexical)
 
 
@@ -208,8 +208,8 @@ def test_categorical_global_ordering_broadcast_rhs(
     s2 = pl.Series("b_cat", ["a"], dtype=pl.Categorical)
     assert_series_equal(op(s, s2), expected_lexical)
 
-    s = s.cast(pl.Categorical("lexical"))
-    s2 = s2.cast(pl.Categorical("lexical"))
+    s = s.cast(pl.Categorical())
+    s2 = s2.cast(pl.Categorical())
     assert_series_equal(op(s, s2), expected_lexical)
     assert_series_equal(op(s, s2.cast(pl.String)), expected_lexical)
 
@@ -234,8 +234,8 @@ def test_categorical_global_ordering_broadcast_lhs(
     s2 = pl.Series(["c", "a", "b"], dtype=pl.Categorical)
     assert_series_equal(op(s, s2), expected_lexical)
 
-    s = s.cast(pl.Categorical("lexical"))
-    s2 = s2.cast(pl.Categorical("lexical"))
+    s = s.cast(pl.Categorical())
+    s2 = s2.cast(pl.Categorical())
     assert_series_equal(op(s, s2), expected_lexical)
     assert_series_equal(op(s, s2.cast(pl.String)), expected_lexical)
 
@@ -432,7 +432,7 @@ def test_categorical_sort_multiple() -> None:
         }
     )
 
-    result = df.with_columns(pl.col("x").cast(pl.Categorical("lexical"))).sort("n", "x")
+    result = df.with_columns(pl.col("x").cast(pl.Categorical())).sort("n", "x")
     assert result["x"].to_list() == ["bar", "baz", "foo"]
 
 
@@ -686,13 +686,13 @@ def test_sort_categorical_retain_none() -> None:
 
 
 def test_cat_preserve_lexical_ordering_on_clear() -> None:
-    s = pl.Series("a", ["a", "b"], dtype=pl.Categorical(ordering="lexical"))
+    s = pl.Series("a", ["a", "b"], dtype=pl.Categorical())
     s2 = s.clear()
     assert s.dtype == s2.dtype
 
 
 def test_cat_preserve_lexical_ordering_on_concat() -> None:
-    dtype = pl.Categorical(ordering="lexical")
+    dtype = pl.Categorical()
 
     df = pl.DataFrame({"x": ["b", "a", "c"]}).with_columns(pl.col("x").cast(dtype))
     df2 = pl.concat([df, df])
@@ -703,13 +703,13 @@ def test_cat_preserve_lexical_ordering_on_concat() -> None:
 @pytest.mark.may_fail_auto_streaming
 def test_cat_append_lexical_sorted_flag() -> None:
     df = pl.DataFrame({"x": [0, 1, 1], "y": ["B", "B", "A"]}).with_columns(
-        pl.col("y").cast(pl.Categorical(ordering="lexical"))
+        pl.col("y").cast(pl.Categorical())
     )
     df2 = pl.concat([part.sort("y") for part in df.partition_by("x")])
 
     assert not (df2["y"].is_sorted())
 
-    s = pl.Series("a", ["z", "k", "a"], pl.Categorical("lexical"))
+    s = pl.Series("a", ["z", "k", "a"], pl.Categorical())
     s1 = s[[0]]
     s2 = s[[1]]
     s3 = s[[2]]
@@ -816,7 +816,7 @@ def test_categorical_prefill() -> None:
 def test_categorical_min_max() -> None:
     schema = pl.Schema(
         {
-            "b": pl.Categorical("lexical"),
+            "b": pl.Categorical(),
             "c": pl.Enum(["foo", "bar"]),
         }
     )
@@ -951,8 +951,8 @@ def test_categorical_serialization_prunes_unused_categories_24034() -> None:
         schema={"a": cat_dtype},
     )
 
-    lf.sink_ipc(f := io.BytesIO())
-    lf_repeat_100.sink_ipc(f_repeat_100 := io.BytesIO())
+    lf.collect().write_ipc(f := io.BytesIO())
+    lf_repeat_100.collect().write_ipc(f_repeat_100 := io.BytesIO())
     ipc_bytes = f.getvalue()
     ipc_repeat_100_bytes = f_repeat_100.getvalue()
 
@@ -961,8 +961,8 @@ def test_categorical_serialization_prunes_unused_categories_24034() -> None:
     ipc_stream_bytes = f.getvalue()
     ipc_stream_repeat_100_bytes = f_repeat_100.getvalue()
 
-    lf.sink_parquet(f := io.BytesIO())
-    lf_repeat_100.sink_parquet(f_repeat_100 := io.BytesIO())
+    lf.collect().write_parquet(f := io.BytesIO())
+    lf_repeat_100.collect().write_parquet(f_repeat_100 := io.BytesIO())
     parquet_bytes = f.getvalue()
     parquet_repeat_100_bytes = f_repeat_100.getvalue()
 
