@@ -1,15 +1,19 @@
 from __future__ import annotations
 
+import itertools
+import random
 import warnings
 from datetime import date, datetime, timedelta
 from typing import TYPE_CHECKING, Any
 
 import numpy as np
 import pytest
+from hypothesis import given
 
 import polars as pl
 from polars.exceptions import DuplicateError, InvalidOperationError
 from polars.testing import assert_frame_equal
+from polars.testing.parametric.strategies.core import dataframes
 
 if TYPE_CHECKING:
     from polars._typing import AsofJoinStrategy, PolarsIntegerType
@@ -518,6 +522,73 @@ def test_asof_join_nearest() -> None:
     assert_frame_equal(out, expected)
 
 
+@pytest.mark.slow
+@pytest.mark.parametrize("allow_exact_matches", [True, False])
+def test_asof_join_nearest_reference(allow_exact_matches: bool) -> None:
+    def asof_join_nearest_reference(
+        df_left: pl.DataFrame, df_right: pl.DataFrame
+    ) -> pl.DataFrame:
+        schema = {"key": pl.Int32, "value": pl.Int32, "value_right": pl.Int32}
+        result = pl.DataFrame(schema=schema)
+        for left_row in df_left.iter_rows():
+            cross_product_sorted = (
+                pl.DataFrame(
+                    {
+                        "key": pl.Series([left_row[0]] * len(df_right), dtype=pl.Int32),
+                        "value": pl.Series(
+                            [left_row[1]] * len(df_right), dtype=pl.Int32
+                        ),
+                        "key_right": df_right["key"],
+                        "value_right": df_right["value"],
+                    },
+                )
+                .with_row_index()
+                .filter(
+                    pl.when(allow_exact_matches)
+                    .then(pl.lit(True))
+                    .otherwise(pl.col("key") != pl.col("key_right"))
+                )
+                .sort(
+                    (pl.col("key") - pl.col("key_right")).abs(),
+                    -pl.col("index").cast(pl.Int32),
+                )
+                .drop("index", "key_right")
+            )
+            if len(cross_product_sorted) == 0:
+                result = result.vstack(
+                    pl.DataFrame([left_row + (None,)], schema=schema, orient="row"),
+                )
+            else:
+                best_match = cross_product_sorted[0]
+                result = result.vstack(best_match)
+        return result
+
+    test_dfs = []
+    rng = random.Random()
+    for n_a, n_b, n_c, n_d in itertools.product([0, 1, 2], repeat=4):
+        a = rng.randint(0, 10)
+        b = rng.randint(0, 10)
+        c = rng.randint(0, 10)
+        d = rng.randint(0, 10)
+        keys = [a] * n_a + [b] * n_b + [c] * n_c + [d] * n_d
+        values = [rng.randint(0, 100000) for _ in keys]
+
+        df = pl.DataFrame(
+            {"key": keys, "value": values}, schema={"key": pl.Int32, "value": pl.Int32}
+        ).sort(by="key")
+        test_dfs.append(df)
+
+    for df_left, df_right in itertools.product(test_dfs, repeat=2):
+        expected = asof_join_nearest_reference(df_left, df_right)
+        actual = df_left.join_asof(
+            df_right,
+            on="key",
+            strategy="nearest",
+            allow_exact_matches=allow_exact_matches,
+        )
+        assert_frame_equal(actual, expected)
+
+
 def test_asof_join_nearest_with_tolerance() -> None:
     a = b = [1, 2, 3, 4, 5]
 
@@ -1287,6 +1358,33 @@ def test_join_asof_no_exact_matches() -> None:
     }
 
 
+@pytest.mark.parametrize("strategy", ["backward", "forward", "nearest"])
+@given(
+    df_left=dataframes(cols=1, allowed_dtypes=pl.Int32),
+    df_right=dataframes(cols=1, allowed_dtypes=pl.Int32),
+)
+def test_join_asof_no_exact_matches_parametric(
+    strategy: AsofJoinStrategy, df_left: pl.DataFrame, df_right: pl.DataFrame
+) -> None:
+    df_left = df_left.sort("col0")
+    df_right = df_right.sort("col0")
+
+    out = df_left.join_asof(
+        df_right,
+        on="col0",
+        strategy=strategy,
+        suffix="_right",
+        coalesce=False,
+        allow_exact_matches=False,
+    )
+
+    for l_val, r_val in zip(
+        out["col0"],
+        out["col0_right"],
+    ):
+        assert l_val != r_val or r_val is None
+
+
 def test_join_asof_not_sorted() -> None:
     df = pl.DataFrame({"a": [1, 1, 1, 2, 2, 2], "b": [2, 1, 3, 1, 2, 3]})
     with pytest.raises(InvalidOperationError, match="is not sorted"):
@@ -1509,3 +1607,32 @@ def test_join_asof_on_cast_expr_24999(by: str | None) -> None:
         expect = expect.with_columns(pl.col("one").alias("one_right"))
 
     assert_frame_equal(q.collect(), expect)
+
+
+@pytest.mark.parametrize(
+    ("left", "right", "expected_right"),
+    [
+        ([], [], []),
+        ([1], [2], [2]),
+        ([], [1, 2], []),
+        ([1, 2], [], [None, None]),
+        ([1, 2], [1, 2], [2, 1]),
+        ([1, 1], [1, 1], [None, None]),
+        ([1, 2, 2, 2, 3], [1, 2, 2, 2, 3], [2, 3, 3, 3, 2]),
+    ],
+)
+def test_join_asof_nearest_no_exact_matches_25468(
+    left: list[int],
+    right: list[int],
+    expected_right: list[int],
+) -> None:
+    df_left = pl.DataFrame({"a": left}, schema={"a": pl.Int32})
+    df_right = pl.DataFrame({"a": right}, schema={"a": pl.Int32})
+    expected_df = pl.DataFrame(
+        {"a": left, "a_right": expected_right},
+        schema={"a": pl.Int32, "a_right": pl.Int32},
+    )
+    result = df_left.join_asof(
+        df_right, on="a", strategy="nearest", coalesce=False, allow_exact_matches=False
+    )
+    assert_frame_equal(result, expected_df)
